﻿using System;
using System.Collections.ObjectModel;
using System.Drawing;
using System.Drawing.Printing;
using System.Windows.Forms;

using ThinkGeo.MapSuite;
using ThinkGeo.MapSuite.Drawing;
using ThinkGeo.MapSuite.Layers;
using ThinkGeo.MapSuite.Shapes;
using ThinkGeo.MapSuite.Styles;
using ThinkGeo.MapSuite.WinForms;

namespace MapSuiteExample
{
    public partial class PrintForm : Form
    {
        PagePrinterLayer pagePrinterLayer;
        PrinterInteractiveOverlay interactiveOverlay;
        MapPrinterLayer mapPrinterLayer;
        PagePrinterLayer marginPrinterLayer;
        const int MARGIN = 1;

        public RectangleShape ShpExtent { get; set; }
        public LayerOverlay Overlay { get; set; }

        public PrintForm()
        {
            InitializeComponent();
        }

        private void PrintForm_Load(object sender, EventArgs e)
        {
            interactiveOverlay = new PrinterInteractiveOverlay();
            winformsMap1.Overlays.Add(interactiveOverlay);
            winformsMap1.BackColor = Color.Gray;

            int dpi = 600;

            //set up pageprinterlayer
            pagePrinterLayer = new PagePrinterLayer(PrinterPageSize.Custom, PrinterOrientation.Landscape);
            pagePrinterLayer.CustomHeight = (float)PrinterHelper.ConvertLength(297, PrintingUnit.Millimeter, PrintingUnit.Inch) * dpi;
            pagePrinterLayer.CustomWidth = (float)PrinterHelper.ConvertLength(210, PrintingUnit.Millimeter, PrintingUnit.Inch) * dpi;
            pagePrinterLayer.BackgroundMask = new AreaStyle(new GeoSolidBrush(GeoColors.White));
            interactiveOverlay.PrinterLayers.Add(pagePrinterLayer);

            //set up one inch margin
            marginPrinterLayer = new PagePrinterLayer(PrinterPageSize.Custom, PrinterOrientation.Landscape);
            marginPrinterLayer.CustomHeight = (float)(PrinterHelper.ConvertLength(297, PrintingUnit.Millimeter, PrintingUnit.Inch) - 1) * dpi;
            marginPrinterLayer.CustomWidth = (float)(PrinterHelper.ConvertLength(210, PrintingUnit.Millimeter, PrintingUnit.Inch) - 1) * dpi;
            marginPrinterLayer.BackgroundMask = new AreaStyle(new GeoPen(GeoColors.Black, 2), new GeoSolidBrush(GeoColors.White));
            interactiveOverlay.PrinterLayers.Add(marginPrinterLayer);

            // set up map printer layer
            mapPrinterLayer = new MapPrinterLayer();
            mapPrinterLayer.MapUnit = GeographyUnit.Meter;
            mapPrinterLayer.BackgroundMask = new AreaStyle(new GeoPen(GeoColor.StandardColors.Black, 1));
            mapPrinterLayer.MapExtent = ShpExtent;
            mapPrinterLayer.DragMode = PrinterDragMode.Fixed;
            mapPrinterLayer.ResizeMode = PrinterResizeMode.Fixed;

            pagePrinterLayer.Open();
            RectangleShape pageBoundingbox = marginPrinterLayer.GetPosition(PrintingUnit.Point);
            mapPrinterLayer.SetPosition(
                pageBoundingbox.Width,
                pageBoundingbox.Height,
                pageBoundingbox.GetCenterPoint().X,
                pageBoundingbox.GetCenterPoint().Y,
                PrintingUnit.Point);

            foreach (Layer layer in Overlay.Layers)
                mapPrinterLayer.Layers.Add(layer);

            marginPrinterLayer.Open();
            mapPrinterLayer.Open();
            interactiveOverlay.PrinterLayers.Add(mapPrinterLayer);

            ScaleBarPrinterLayer scale = new ScaleBarPrinterLayer(mapPrinterLayer);
            scale.UnitFamily = UnitSystem.Metric;
            scale.BackgroundMask = new AreaStyle(new GeoSolidBrush(GeoColors.White));
            scale.SetPosition(3, 0.5, LayerAnchorLocation.LowerLeft, pageBoundingbox.LowerLeftPoint.X, pageBoundingbox.LowerLeftPoint.Y, PrintingUnit.Point);
            scale.Open();
            interactiveOverlay.PrinterLayers.Add(scale);


            winformsMap1.Overlays.Add(interactiveOverlay);
            winformsMap1.CurrentExtent = pagePrinterLayer.GetBoundingBox();
            winformsMap1.Refresh();
        }

        private void button1_Click(object sender, EventArgs e)
        {
            //PrintDialog settings
            PrintDialog printDlg = new PrintDialog();

            // (a)(1) Set up the print document
            PrintDocument printDocument = new PrintDocument();

            printDlg.Document = printDocument;
            printDlg.AllowSelection = false;
            printDlg.AllowSomePages = false;
            printDlg.Document.DefaultPageSettings.Landscape = pagePrinterLayer.Orientation == PrinterOrientation.Landscape;
            printDlg.Document.DefaultPageSettings.PaperSize = new PaperSize("A4", (int)pagePrinterLayer.CustomWidth, (int)pagePrinterLayer.CustomHeight);
            printDlg.Document.DefaultPageSettings.Margins = new Margins();
            printDlg.Document.OriginAtMargins = false;

            if (printDlg.ShowDialog() == DialogResult.OK)
            {
                // (b) Set up PrinterGeoCanvas
                PrinterGeoCanvas printerGeoCanvas = new PrinterGeoCanvas();
                printerGeoCanvas.Dpi = 100;
                printerGeoCanvas.DrawingArea = printDlg.Document.DefaultPageSettings.Bounds;
                printerGeoCanvas.DrawingQuality = DrawingQuality.HighQuality;
                printerGeoCanvas.ClippingArea = pagePrinterLayer.GetBoundingBox();

                var simpleCandidates = new Collection<SimpleCandidate>();
                printerGeoCanvas.BeginDrawing(printDlg.Document, pagePrinterLayer.GetBoundingBox(), mapPrinterLayer.MapUnit);

                foreach (PrinterLayer layer in interactiveOverlay.PrinterLayers)
                {
                    layer.Draw(printerGeoCanvas, simpleCandidates);
                }

                printerGeoCanvas.EndDrawing();
            }
        }


    }
}
